@@ -27,9 +27,9 @@
  *
  * Contributor(s): /
  *
- * $Revision: 20385 $
+ * $Revision: 21996 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 10:40:38 +0000 (Wed, 04 Jun 2008) $
+ * $Date: 2009-02-05 00:26:50 +0000 (Thu, 05 Feb 2009) $
  */
 
 #pragma implementation "sound_alsa.h"
@@ -48,7 +48,8 @@ PMutex dictionaryMutex;
 
 PSoundChannelALSA::PSoundChannelALSA()
 {
-  PSoundChannelALSA::Construct();
+  card_nr = 0;
+  os_handle = NULL;
 }
 
 
@@ -58,7 +59,8 @@ PSoundChannelALSA::PSoundChannelALSA (const PString &device,
                                             unsigned sampleRate,
                                             unsigned bitsPerSample)
 {
-  Construct();
+  card_nr = 0;
+  os_handle = NULL;
   Open (device, dir, numChannels, sampleRate, bitsPerSample);
 }
 
@@ -79,6 +81,7 @@ void PSoundChannelALSA::Construct()
 
   card_nr = 0;
   os_handle = NULL;
+  isInitialised = PFalse;
 }
 
 
@@ -90,72 +93,60 @@ PSoundChannelALSA::~PSoundChannelALSA()
 
 void PSoundChannelALSA::UpdateDictionary (Directions dir)
 {
-  int card = -1, dev = -1;
-  
-  snd_ctl_t *handle = NULL;
-  snd_ctl_card_info_t *info = NULL;
-  snd_pcm_info_t *pcminfo = NULL;
-  snd_pcm_stream_t stream;
+  PWaitAndSignal mutex(dictionaryMutex);
 
-  char *name = NULL;
-  char card_id [32];
+  PStringToOrdinal & devices = dir == Recorder ? capture_devices : playback_devices;
+  devices.RemoveAll();
 
-  PWaitAndSignal m(dictionaryMutex);
+  int cardNum = -1;
+  if (snd_card_next(&cardNum) < 0 || cardNum < 0)
+    return;  // No sound card found
 
-  if (dir == Recorder) {
+  snd_ctl_card_info_t * info = NULL;
+  snd_ctl_card_info_alloca(&info);
 
-    stream = SND_PCM_STREAM_CAPTURE;
-    capture_devices = PStringToOrdinal ();
-  }
-  else {
+  snd_pcm_info_t * pcminfo = NULL;
+  snd_pcm_info_alloca(&pcminfo);
 
-    stream = SND_PCM_STREAM_PLAYBACK;
-    playback_devices = PStringToOrdinal ();
-  }
+  do {
+    char card_id[32];
+    snprintf(card_id, sizeof(card_id), "hw:%d", cardNum);
 
-  snd_ctl_card_info_alloca (&info);
-  snd_pcm_info_alloca (&pcminfo);
-
-  /* No sound card found */
-  if (snd_card_next (&card) < 0 || card < 0) {
-
-    return;
-  }
-
-  while (card >= 0) {
-
-    snprintf (card_id, 32, "hw:%d", card);
-    
-    if (snd_ctl_open (&handle, card_id, 0) == 0) {
-
-      snd_ctl_card_info (handle, info);
-
-      while (1) {
-
-        snd_ctl_pcm_next_device (handle, &dev);
+    snd_ctl_t * handle = NULL;
+    if (snd_ctl_open(&handle, card_id, 0) == 0) {
+      snd_ctl_card_info(handle, info);
 
+      int dev = -1;
+      for (;;) {
+        snd_ctl_pcm_next_device(handle, &dev);
         if (dev < 0)
           break;
 
-        snd_pcm_info_set_device (pcminfo, dev);
-        snd_pcm_info_set_subdevice (pcminfo, 0);
-        snd_pcm_info_set_stream (pcminfo, stream);
+        snd_pcm_info_set_device(pcminfo, dev);
+        snd_pcm_info_set_subdevice(pcminfo, 0);
+        snd_pcm_info_set_stream(pcminfo, dir == Recorder ? SND_PCM_STREAM_CAPTURE : SND_PCM_STREAM_PLAYBACK);
 
-        if (snd_ctl_pcm_info (handle, pcminfo) >= 0) {
+        if (snd_ctl_pcm_info(handle, pcminfo) >= 0) {
+          char * rawName = NULL;
+          snd_card_get_name(cardNum, &rawName);
+          if (rawName != NULL) {
+            int disambiguator = 1;
+            PString uniqueName = rawName;
+            while (devices.Contains(uniqueName)) {
+              uniqueName = rawName;
+              uniqueName.sprintf(" (%d)", disambiguator++);
+            }
 
-          snd_card_get_name (card, &name);
-          if (dir == Recorder) 
-            capture_devices.SetAt (name, card);
-          else 
-            playback_devices.SetAt (name, card);
-          free (name);
+            devices.SetAt(uniqueName, cardNum);
+            free(rawName);
+          }
         }
       }
       snd_ctl_close(handle);
     }
 
-    snd_card_next (&card);
-  }
+    snd_card_next(&cardNum);
+  } while (cardNum >= 0);
 }
 
 
@@ -209,9 +200,8 @@ PBoolean PSoundChannelALSA::Open (const PString & _device,
   mNumChannels = _numChannels;
   mSampleRate = _sampleRate;
   mBitsPerSample = _bitsPerSample;
-  isInitialised = PFalse;
 
-  os_handle = NULL;
+  Construct();
 
   PWaitAndSignal m(device_mutex);
 
@@ -427,8 +417,9 @@ PBoolean PSoundChannelALSA::Write (const void *buf, PINDEX len)
     else {
       if (r == -EPIPE) {    /* under-run */
         r = snd_pcm_prepare (os_handle);
-        if (r < 0)
+        if (r < 0) {
           PTRACE (1, "ALSA\tCould not prepare device: " << snd_strerror (r));
+        }
       } else if (r == -ESTRPIPE) {
 
         while ((r = snd_pcm_resume (os_handle)) == -EAGAIN)
