@@ -23,9 +23,9 @@
  *
  * Copyright 2003 Equivalence Pty. Ltd.
  *
- * $Revision: 20385 $
+ * $Revision: 22295 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2009-03-26 00:31:31 +0000 (Thu, 26 Mar 2009) $
  */
 
 #ifdef __GNUC__
@@ -191,17 +191,24 @@ static PMutex & GetDNSMutex()
 DNS_STATUS DnsQuery_A(const char * service,
                               WORD requestType,
                              DWORD options,
-                            void *,
+                            PIP4_ARRAY,
                      PDNS_RECORD * results,
                             void *)
 {
+#if defined(P_NETBSD)
+  struct __res_state myRes;
+#endif
   if (results == NULL)
     return -1;
 
   *results = NULL;
 
 #if P_HAS_RES_NINIT
+#if defined(P_NETBSD)
+  res_ninit(&myRes);
+#else
   res_ninit(&_res);
+#endif
 #else
   res_init();
   GetDNSMutex().Wait();
@@ -213,7 +220,13 @@ DNS_STATUS DnsQuery_A(const char * service,
   } reply;
 
 #if P_HAS_RES_NINIT
-  int replyLen = res_nsearch(&_res, service, C_IN, requestType, (BYTE *)&reply, sizeof(reply));
+  int replyLen = res_nsearch(
+#if defined(P_NETBSD)
+      &myRes,
+#else
+      &_res,
+#endif
+      service, C_IN, requestType, (BYTE *)&reply, sizeof(reply));
 #else
   int replyLen = res_search(service, C_IN, requestType, (BYTE *)&reply, sizeof(reply));
   GetDNSMutex().Signal();
@@ -465,7 +478,7 @@ PBoolean PDNS::LookupSRV(
 
   PIPSocketAddressAndPortVector::const_iterator r;
   for (r = info.begin(); r != info.end(); ++r) 
-    returnList.AppendString(user + r->address.AsString() + ":" + PString(PString::Unsigned, r->port));
+    returnList.AppendString(user + r->AsString(':'));
 
   return returnList.GetSize() != 0;;
 }
@@ -505,15 +518,7 @@ PBoolean PDNS::LookupSRV(
     PTRACE(5,"DNS\tSRV Record found " << srvLookupStr);
     PDNS::SRVRecord * recPtr = srvRecords.GetFirst();
     while (recPtr != NULL) {
-      PIPSocketAddressAndPort addrAndPort;
-
-      addrAndPort.address = recPtr->hostAddress;
-
-      if (recPtr->port > 0)
-        addrAndPort.port = recPtr->port;
-      else
-        addrAndPort.port = defaultPort;
-
+      PIPSocketAddressAndPort addrAndPort(recPtr->hostAddress, recPtr->port > 0 ? recPtr->port : defaultPort);
       addrList.push_back(addrAndPort);
 
       recPtr = srvRecords.GetNext();
