@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21788 $
+ * $Revision: 22284 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-03-25 09:44:02 +0000 (Wed, 25 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -535,7 +535,8 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     struct addrinfo hints = { AI_CANONNAME, defaultIpAddressFamily };
     localErrNo = getaddrinfo((const char *)name, NULL , &hints, &res);
     host = new PIPCacheData(localErrNo != NETDB_SUCCESS ? NULL : res, name);
-    freeaddrinfo(res);
+    if (res != NULL)
+      freeaddrinfo(res);
 
 #else // HAS_GETADDRINFO
 
@@ -1236,6 +1237,18 @@ PBoolean PIPSocket::GetLocalAddress(Address & addr)
 }
 
 
+PBoolean PIPSocket::GetLocalAddress(PIPSocketAddressAndPort & addr)
+{
+  Address ip;
+  WORD port;
+  if (!GetLocalAddress(ip, port))
+    return false;
+
+  addr.SetAddress(ip, port);
+  return true;
+}
+
+
 PBoolean PIPSocket::GetLocalAddress(Address & addr, WORD & portNum)
 {
 #if P_HAS_IPV6
@@ -1292,6 +1305,19 @@ PBoolean PIPSocket::GetPeerAddress(Address & addr)
   return GetPeerAddress(addr, portNum);
 }
 
+
+PBoolean PIPSocket::GetPeerAddress(PIPSocketAddressAndPort & addr)
+{
+  Address ip;
+  WORD port;
+  if (!GetPeerAddress(ip, port))
+    return false;
+
+  addr.SetAddress(ip, port);
+  return true;
+}
+
+
 PBoolean PIPSocket::GetPeerAddress(Address & addr, WORD & portNum)
 {
 #if P_HAS_IPV6
@@ -1517,7 +1543,26 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
       return PFalse;
 
     port = sa.GetPort();
-    return PTrue;
+
+    if (!bindAddr.IsMulticast())
+      return true;
+
+    // Probably move this stuff into a separate method
+    if (bindAddr.GetVersion() == 4) {
+      struct ip_mreq  mreq;
+      mreq.imr_multiaddr = in_addr(bindAddr);
+      mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+      if (SetOption(IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq), IPPROTO_IP)) {
+        PTRACE(4, "Socket\tJoined multicast group");
+        return true;
+      }
+      PTRACE(1, "Socket\tFailed to join multicast group");
+    }
+    else {
+      PTRACE(1, "Socket\tIPV6 Multicast join not implemented yet");
+    }
+
+    return false;
   }
 
 #else
@@ -1543,7 +1588,18 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
     socklen_t size = sizeof(sin);
     if (ConvertOSError(::getsockname(os_handle, (struct sockaddr*)&sin, &size))) {
       port = ntohs(sin.sin_port);
-      return PTrue;
+
+	  if (!IN_MULTICAST(ntohl(sin.sin_addr.s_addr)))
+        return true;
+
+      struct ip_mreq  mreq;
+      mreq.imr_multiaddr.s_addr = sin.sin_addr.s_addr;
+      mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+      if (SetOption(IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq))) {
+        PTRACE(4, "Socket\tJoined multicast group");
+        return true;
+      }
+      PTRACE(1, "Socket\tFailed to join multicast group");
     }
   }
 
@@ -1870,7 +1926,8 @@ PBoolean PIPSocket::Address::FromString(const PString & ipAndInterface)
         struct sockaddr_in * addr_in = (struct sockaddr_in *)res->ai_addr;
         v.four = addr_in->sin_addr;
       }
-      freeaddrinfo(res);
+      if (res != NULL)
+        freeaddrinfo(res);
       return IsValid();
     }
 
@@ -2026,6 +2083,18 @@ PBoolean PIPSocket::Address::IsBroadcast() const
   return *this == broadcast4;
 }
 
+
+PBoolean PIPSocket::Address::IsMulticast() const
+{
+#if P_HAS_IPV6
+  if (version == 6)
+    return IN6_IS_ADDR_MULTICAST(&v.six);
+#endif
+
+  return IN_MULTICAST(ntohl(v.four.s_addr));
+}
+
+
 PBoolean PIPSocket::Address::IsRFC1918() const 
 { 
 #if P_HAS_IPV6
@@ -2833,4 +2902,12 @@ PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD port, char sep
 }
 
 
+void PIPSocketAddressAndPort::SetAddress(const PIPSocket::Address & addr, WORD port)
+{
+  m_address = addr;
+  if (port != 0)
+    m_port = port;
+}
+
+
 // End Of File ///////////////////////////////////////////////////////////////
