@@ -23,9 +23,9 @@
  *
  * Contributor(s): Derek J Smithies (derek@indranet.co.nz)
  *
- * $Revision: 21176 $
- * $Author: ms30002000 $
- * $Date: 2008-09-27 20:34:32 +0000 (Sat, 27 Sep 2008) $
+ * $Revision: 21884 $
+ * $Author: csoutheren $
+ * $Date: 2009-01-05 04:26:54 +0000 (Mon, 05 Jan 2009) $
  */
 
 #include <ptlib.h>
@@ -39,11 +39,9 @@
 #include <ptlib/videoio.h>
 #include <ptclib/delaychan.h>
 
-#if defined(P_MACOSX) || defined(P_MINGW)
 namespace PWLibStupidLinkerHacks {
   int loadFakeVideoStuff;
 };
-#endif
 
 enum {
   eMovingBlocks,
