@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20510 $
- * $Author: shorne $
- * $Date: 2008-06-28 05:19:50 +0200 (sam 28 jun 2008) $
+ * $Revision: 22295 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-26 00:31:31 +0000 (Thu, 26 Mar 2009) $
  */
 
 #define _CONFIG_CXX
@@ -49,10 +49,18 @@
 #define	EXTENSION		".ini"
 #define	ENVIRONMENT_CONFIG_STR	"/\~~environment~~\/"
 
+#ifdef P_MACOSX
+#define environ (NULL)
+#endif
+
+#if defined(__NetBSD__)
+extern char **environ;
+#endif
+
 //
 //  a single key/value pair
 //
-PDECLARE_CLASS (PXConfigValue, PCaselessString)
+PDECLARE_CLASS(PXConfigValue, PCaselessString)
   public:
     PXConfigValue(const PString & theKey, const PString & theValue = "") 
       : PCaselessString(theKey), value(theValue) { }
@@ -66,7 +74,7 @@ PDECLARE_CLASS (PXConfigValue, PCaselessString)
 //
 //  a list of key/value pairs
 //
-PLIST (PXConfigSectionList, PXConfigValue);
+PLIST(PXConfigSectionList, PXConfigValue);
 
 //
 //  a list of key value pairs, with a section name
@@ -86,9 +94,12 @@ PDECLARE_CLASS(PXConfigSection, PCaselessString)
 // a list of sections
 //
 
-PDECLARE_LIST(PXConfig, PXConfigSection)
+class PXConfig : public PList<PXConfigSection>
+{
+    PCLASSINFO(PXConfig, PList<PXConfigSection>);
   public:
-    PXConfig(int i = 0);
+    PXConfig();
+    ~PXConfig();
 
     void Wait()   { mutex.Wait(); }
     void Signal() { mutex.Signal(); }
@@ -99,7 +110,11 @@ PDECLARE_LIST(PXConfig, PXConfigSection)
     PBoolean WriteToFile(const PFilePath & filename);
     PBoolean Flush(const PFilePath & filename);
 
-    void SetDirty()   { dirty = PTrue; }
+    void SetDirty()
+    {
+      PTRACE_IF(4, !dirty, "PTLib\tSetting PXConfig dirty.");
+      dirty = PTrue;
+    }
 
     PBoolean      AddInstance();
     PBoolean      RemoveInstance(const PFilePath & filename);
@@ -116,9 +131,12 @@ PDECLARE_LIST(PXConfig, PXConfigSection)
 //
 // a dictionary of configurations, keyed by filename
 //
-PDECLARE_DICTIONARY(PXConfigDictionary, PFilePath, PXConfig)
+typedef PDictionary<PFilePath, PXConfig> PXConfigDictionaryBase;
+class PXConfigDictionary : public PXConfigDictionaryBase
+{
+    PCLASSINFO(PXConfigDictionary, PXConfigDictionaryBase)
   public:
-    PXConfigDictionary(int dummy);
+    PXConfigDictionary();
     ~PXConfigDictionary();
     PXConfig * GetFileConfigInstance(const PFilePath & key, const PFilePath & readKey);
     PXConfig * GetEnvironmentInstance();
@@ -151,7 +169,7 @@ PXConfigDictionary * configDict;
 
 void PProcess::CreateConfigFilesDictionary()
 {
-  configFiles = new PXConfigDictionary(0);
+  configFiles = new PXConfigDictionary;
 }
 
 
@@ -168,6 +186,7 @@ PXConfigWriteThread::~PXConfigWriteThread()
 
 void PXConfigWriteThread::Main()
 {
+  PTRACE(4, "PTLib\tConfig file cache write back thread started.");
   while (!stop.Wait(30000))  // if stop.Wait() returns PTrue, we are shutting down
     configDict->WriteChangedInstances();   // check dictionary for items that need writing
 
@@ -178,7 +197,7 @@ void PXConfigWriteThread::Main()
 
 
 
-PXConfig::PXConfig(int)
+PXConfig::PXConfig()
 {
   // make sure content gets removed
   AllowDeleteObjects();
@@ -191,8 +210,16 @@ PXConfig::PXConfig(int)
 
   // normally save on exit (except for environment configs)
   canSave = PTrue;
+
+  PTRACE(4, "PTLib\tCreated PXConfig " << this);
 }
 
+PXConfig::~PXConfig()
+{
+  PTRACE(4, "PTLib\tDestroyed PXConfig " << this);
+}
+
+
 PBoolean PXConfig::AddInstance()
 {
   mutex.Wait();
@@ -276,7 +303,7 @@ PBoolean PXConfig::WriteToFile(const PFilePath & filename)
     }
   }
 
-  PTRACE(4, "PWLib\tSaved config file: " << filename);
+  PTRACE(4, "PTLib\tSaved config file: " << filename);
   return PTrue;
 }
 
@@ -288,6 +315,8 @@ PBoolean PXConfig::ReadFromFile(const PFilePath & filename)
   // clear out all information
   RemoveAll();
 
+  PTRACE(4, "PTLib\tReading config file: " << filename);
+
   // attempt to open file
   PTextFile file;
   if (!file.Open(filename, PFile::ReadOnly))
@@ -416,7 +445,7 @@ PString PProcess::GetConfigurationFile()
 // PXConfigDictionary
 //
 
-PXConfigDictionary::PXConfigDictionary(int)
+PXConfigDictionary::PXConfigDictionary()
 {
   environmentInstance = NULL;
   writeThread = NULL;
@@ -439,8 +468,8 @@ PXConfig * PXConfigDictionary::GetEnvironmentInstance()
 {
   mutex.Wait();
   if (environmentInstance == NULL) {
-    environmentInstance = new PXConfig(0);
-    environmentInstance->ReadFromEnvironment(PProcess::Current().PXGetEnvp());
+    environmentInstance = new PXConfig;
+    environmentInstance->ReadFromEnvironment(environ);
   }
   mutex.Signal();
   return environmentInstance;
@@ -459,7 +488,7 @@ PXConfig * PXConfigDictionary::GetFileConfigInstance(const PFilePath & key, cons
   if (config != NULL) 
     config->AddInstance();
   else {
-    config = new PXConfig(0);
+    config = new PXConfig;
     config->ReadFromFile(readKey);
     config->AddInstance();
     SetAt(key, config);
@@ -479,8 +508,10 @@ void PXConfigDictionary::RemoveInstance(PXConfig * instance)
 
     // decrement the instance count and remove it if this was the last instance
     PFilePath key = GetKeyAt(index);
-    if (instance->RemoveInstance(key))
-          RemoveAt(key);
+    if (instance->RemoveInstance(key)) {
+      instance->Flush(key);
+      RemoveAt(key);
+    }
   }
 
   mutex.Signal();
@@ -538,18 +569,19 @@ PConfig::PConfig(int, const PString & name)
   config = configDict->GetFileConfigInstance(filename, readFilename);
 }
 
-void PConfig::Construct(const PFilePath & theFilename)
 
+void PConfig::Construct(const PFilePath & theFilename)
 {
   config = configDict->GetFileConfigInstance(theFilename, theFilename);
 }
 
-PConfig::~PConfig()
 
+PConfig::~PConfig()
 {
   configDict->RemoveInstance(config);
 }
 
+
 ////////////////////////////////////////////////////////////
 //
 // PConfig::
