@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20769 $
- * $Author: csoutheren $
- * $Date: 2008-08-30 00:00:24 +0000 (Sat, 30 Aug 2008) $
+ * $Revision: 21137 $
+ * $Author: ms30002000 $
+ * $Date: 2008-09-22 16:57:19 +0000 (Mon, 22 Sep 2008) $
  */
 
 #pragma implementation "remconn.h"
@@ -49,7 +49,7 @@
 #include <signal.h>
 #endif
 
-#ifdef __BEOS__
+#ifdef P_BEOS
 #include <signal.h>
 #endif
 
