@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21137 $
- * $Author: ms30002000 $
- * $Date: 2008-09-22 16:57:19 +0000 (Mon, 22 Sep 2008) $
+ * $Revision: 22356 $
+ * $Author: rjongbloed $
+ * $Date: 2009-03-31 23:57:19 +0000 (Tue, 31 Mar 2009) $
  */
 
 #define _OSUTIL_CXX
@@ -399,8 +399,8 @@ extern PProcess * PProcessInstance;;
 PString PX_GetThreadName(pthread_t id)
 {
   if (PProcessInstance != NULL) {
-    PWaitAndSignal m(PProcessInstance->threadMutex);
-    PThread & thread = PProcessInstance->activeThreads[(unsigned)id];
+    PWaitAndSignal m(PProcessInstance->activeThreadMutex);
+    PThread & thread = PProcessInstance->activeThreads[(uintptr_t)id];
     return thread.GetThreadName();
   }
   return psprintf("%08x", id);
@@ -503,6 +503,10 @@ void PProcess::PXOnAsyncSignal(int sig)
   switch (sig) {
     case SIGINT:
     case SIGTERM:
+      if (OnInterrupt(sig == SIGTERM))
+        break;
+      // Do next case
+
     case SIGHUP:
       raise(SIGKILL);
       break;
@@ -524,7 +528,7 @@ void PProcess::PXOnSignal(int sig)
     PMemoryHeap::GetState(state);
 #endif
     PStringStream strm;
-    threadMutex.Wait();
+    activeThreadMutex.Wait();
     PINDEX i;
     strm << "===============\n";
     strm << activeThreads.GetSize() << " active threads\n";
@@ -539,7 +543,7 @@ void PProcess::PXOnSignal(int sig)
     PMemoryHeap::GetState(state);
 #endif
     strm << "===============\n";
-    threadMutex.Signal();
+    activeThreadMutex.Signal();
     fprintf(stderr, "%s", strm.GetPointer());
 #if PMEMORY_CHECK
     PMemoryHeap::SetIgnoreAllocations(oldIgnore);
@@ -553,7 +557,8 @@ void PProcess::CommonConstruct()
   // Setup signal handlers
   pxSignals = 0;
 
-  SetSignals(&PXSignalHandler);
+  if (!m_library)
+    SetSignals(&PXSignalHandler);
 
 #if !defined(P_VXWORKS) && !defined(P_RTEMS)
   // initialise the timezone information
@@ -571,7 +576,9 @@ void PProcess::CommonDestruct()
   delete configFiles;
 #endif
   configFiles = NULL;
-  SetSignals(NULL);
+
+  if (!m_library)
+    SetSignals(NULL);
 }
 
 // rtems fixes
